import BridgeIDBObjectStore from "./BridgeIDBObjectStore";
import BridgeIDBRequest from "./BridgeIDBRequest";
import { CursorRange, CursorSource, Key, Value, BridgeIDBCursorDirection } from "./util/types";
import { Backend } from "./backend-interface";
/**
 * http://www.w3.org/TR/2015/REC-IndexedDB-20150108/#cursor
 */
export declare class BridgeIDBCursor {
    _request: BridgeIDBRequest | undefined;
    private _gotValue;
    private _range;
    private _indexPosition;
    private _objectStorePosition;
    private _keyOnly;
    private _source;
    private _direction;
    private _key;
    private _primaryKey;
    private _indexName;
    private _objectStoreName;
    protected _value: Value;
    constructor(source: CursorSource, objectStoreName: string, indexName: string | undefined, range: CursorRange, direction: BridgeIDBCursorDirection, request: BridgeIDBRequest, keyOnly: boolean);
    get _effectiveObjectStore(): BridgeIDBObjectStore;
    get _backend(): Backend;
    get source(): CursorSource;
    set source(val: CursorSource);
    get direction(): BridgeIDBCursorDirection;
    set direction(val: BridgeIDBCursorDirection);
    get key(): undefined;
    set key(val: undefined);
    get primaryKey(): any;
    set primaryKey(val: any);
    protected get _isValueCursor(): boolean;
    /**
     * https://w3c.github.io/IndexedDB/#iterate-a-cursor
     */
    _iterate(key?: Key, primaryKey?: Key): Promise<any>;
    update(value: Value): BridgeIDBRequest;
    /**
     * http://www.w3.org/TR/2015/REC-IndexedDB-20150108/#widl-IDBCursor-advance-void-unsigned-long-count
     */
    advance(count: number): void;
    /**
     * http://www.w3.org/TR/2015/REC-IndexedDB-20150108/#widl-IDBCursor-continue-void-any-key
     */
    continue(key?: Key): void;
    continuePrimaryKey(key: Key, primaryKey: Key): void;
    delete(): BridgeIDBRequest;
    toString(): string;
}
export default BridgeIDBCursor;
