"use strict";
/*
 Copyright 2019 Florian Dold

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 or implied. See the License for the specific language governing
 permissions and limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var ava_1 = require("ava");
var MemoryBackend_1 = require("./MemoryBackend");
var BridgeIDBFactory_1 = require("./BridgeIDBFactory");
var BridgeIDBKeyRange_1 = require("./BridgeIDBKeyRange");
function promiseFromRequest(request) {
    return new Promise(function (resolve, reject) {
        request.onsuccess = function () {
            resolve(request.result);
        };
        request.onerror = function () {
            reject(request.error);
        };
    });
}
function promiseFromTransaction(transaction) {
    return new Promise(function (resolve, reject) {
        transaction.oncomplete = function () {
            resolve();
        };
        transaction.onerror = function () {
            reject();
        };
    });
}
ava_1.default("Spec: Example 1 Part 1", function (t) { return __awaiter(void 0, void 0, void 0, function () {
    var backend, idb, request;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                backend = new MemoryBackend_1.default();
                idb = new BridgeIDBFactory_1.default(backend);
                request = idb.open("library");
                request.onupgradeneeded = function () {
                    var db = request.result;
                    var store = db.createObjectStore("books", { keyPath: "isbn" });
                    var titleIndex = store.createIndex("by_title", "title", { unique: true });
                    var authorIndex = store.createIndex("by_author", "author");
                    // Populate with initial data.
                    store.put({ title: "Quarry Memories", author: "Fred", isbn: 123456 });
                    store.put({ title: "Water Buffaloes", author: "Fred", isbn: 234567 });
                    store.put({ title: "Bedrock Nights", author: "Barney", isbn: 345678 });
                };
                return [4 /*yield*/, promiseFromRequest(request)];
            case 1:
                _a.sent();
                t.pass();
                return [2 /*return*/];
        }
    });
}); });
ava_1.default("Spec: Example 1 Part 2", function (t) { return __awaiter(void 0, void 0, void 0, function () {
    var backend, idb, request, db, tx, store;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                backend = new MemoryBackend_1.default();
                idb = new BridgeIDBFactory_1.default(backend);
                request = idb.open("library");
                request.onupgradeneeded = function () {
                    var db = request.result;
                    var store = db.createObjectStore("books", { keyPath: "isbn" });
                    var titleIndex = store.createIndex("by_title", "title", { unique: true });
                    var authorIndex = store.createIndex("by_author", "author");
                };
                return [4 /*yield*/, promiseFromRequest(request)];
            case 1:
                db = _a.sent();
                t.is(db.name, "library");
                tx = db.transaction("books", "readwrite");
                tx.oncomplete = function () {
                    console.log("oncomplete called");
                };
                store = tx.objectStore("books");
                store.put({ title: "Quarry Memories", author: "Fred", isbn: 123456 });
                store.put({ title: "Water Buffaloes", author: "Fred", isbn: 234567 });
                store.put({ title: "Bedrock Nights", author: "Barney", isbn: 345678 });
                return [4 /*yield*/, promiseFromTransaction(tx)];
            case 2:
                _a.sent();
                t.pass();
                return [2 /*return*/];
        }
    });
}); });
ava_1.default("Spec: Example 1 Part 3", function (t) { return __awaiter(void 0, void 0, void 0, function () {
    var backend, idb, request, db, tx, store, tx2, store2, index2, request2, result2, tx3, store3, index3, request3, cursor, tx4, store4, request4, tx5, store5, index5, request5, request6, request7;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                backend = new MemoryBackend_1.default();
                idb = new BridgeIDBFactory_1.default(backend);
                request = idb.open("library");
                request.onupgradeneeded = function () {
                    var db = request.result;
                    var store = db.createObjectStore("books", { keyPath: "isbn" });
                    var titleIndex = store.createIndex("by_title", "title", { unique: true });
                    var authorIndex = store.createIndex("by_author", "author");
                };
                return [4 /*yield*/, promiseFromRequest(request)];
            case 1:
                db = _a.sent();
                t.is(db.name, "library");
                tx = db.transaction("books", "readwrite");
                store = tx.objectStore("books");
                store.put({ title: "Bedrock Nights", author: "Barney", isbn: 345678 });
                store.put({ title: "Quarry Memories", author: "Fred", isbn: 123456 });
                store.put({ title: "Water Buffaloes", author: "Fred", isbn: 234567 });
                return [4 /*yield*/, promiseFromTransaction(tx)];
            case 2:
                _a.sent();
                tx2 = db.transaction("books", "readonly");
                store2 = tx2.objectStore("books");
                index2 = store2.index("by_title");
                request2 = index2.get("Bedrock Nights");
                return [4 /*yield*/, promiseFromRequest(request2)];
            case 3:
                result2 = _a.sent();
                t.is(result2.author, "Barney");
                tx3 = db.transaction(["books"], "readonly");
                store3 = tx3.objectStore("books");
                index3 = store3.index("by_author");
                request3 = index3.openCursor(BridgeIDBKeyRange_1.default.only("Fred"));
                return [4 /*yield*/, promiseFromRequest(request3)];
            case 4:
                _a.sent();
                cursor = request3.result;
                t.is(cursor.value.author, "Fred");
                t.is(cursor.value.isbn, 123456);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request3)];
            case 5:
                _a.sent();
                cursor = request3.result;
                t.is(cursor.value.author, "Fred");
                t.is(cursor.value.isbn, 234567);
                return [4 /*yield*/, promiseFromTransaction(tx3)];
            case 6:
                _a.sent();
                tx4 = db.transaction("books", "readonly");
                store4 = tx4.objectStore("books");
                request4 = store4.openCursor();
                return [4 /*yield*/, promiseFromRequest(request4)];
            case 7:
                _a.sent();
                cursor = request4.result;
                t.is(cursor.value.isbn, 123456);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request4)];
            case 8:
                _a.sent();
                cursor = request4.result;
                t.is(cursor.value.isbn, 234567);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request4)];
            case 9:
                _a.sent();
                cursor = request4.result;
                t.is(cursor.value.isbn, 345678);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request4)];
            case 10:
                _a.sent();
                cursor = request4.result;
                t.is(cursor, null);
                tx5 = db.transaction("books", "readonly");
                store5 = tx5.objectStore("books");
                index5 = store5.index("by_author");
                request5 = index5.openCursor(null, "next");
                return [4 /*yield*/, promiseFromRequest(request5)];
            case 11:
                _a.sent();
                cursor = request5.result;
                t.is(cursor.value.author, "Barney");
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request5)];
            case 12:
                _a.sent();
                cursor = request5.result;
                t.is(cursor.value.author, "Fred");
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request5)];
            case 13:
                _a.sent();
                cursor = request5.result;
                t.is(cursor.value.author, "Fred");
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request5)];
            case 14:
                _a.sent();
                cursor = request5.result;
                t.is(cursor, null);
                request6 = index5.openCursor(null, "nextunique");
                return [4 /*yield*/, promiseFromRequest(request6)];
            case 15:
                _a.sent();
                cursor = request6.result;
                t.is(cursor.value.author, "Barney");
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request6)];
            case 16:
                _a.sent();
                cursor = request6.result;
                t.is(cursor.value.author, "Fred");
                t.is(cursor.value.isbn, 123456);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request6)];
            case 17:
                _a.sent();
                cursor = request6.result;
                t.is(cursor, null);
                request7 = index5.openCursor(null, "prevunique");
                return [4 /*yield*/, promiseFromRequest(request7)];
            case 18:
                _a.sent();
                cursor = request7.result;
                t.is(cursor.value.author, "Fred");
                t.is(cursor.value.isbn, 234567);
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request7)];
            case 19:
                _a.sent();
                cursor = request7.result;
                t.is(cursor.value.author, "Barney");
                cursor.continue();
                return [4 /*yield*/, promiseFromRequest(request7)];
            case 20:
                _a.sent();
                cursor = request7.result;
                t.is(cursor, null);
                db.close();
                t.pass();
                return [2 /*return*/];
        }
    });
}); });
ava_1.default("simple deletion", function (t) { return __awaiter(void 0, void 0, void 0, function () {
    var backend, idb, request, db, tx, store, tx2, store2, req1, req2, req3;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                backend = new MemoryBackend_1.default();
                idb = new BridgeIDBFactory_1.default(backend);
                request = idb.open("library");
                request.onupgradeneeded = function () {
                    var db = request.result;
                    var store = db.createObjectStore("books", { keyPath: "isbn" });
                    var titleIndex = store.createIndex("by_title", "title", { unique: true });
                    var authorIndex = store.createIndex("by_author", "author");
                };
                return [4 /*yield*/, promiseFromRequest(request)];
            case 1:
                db = _a.sent();
                t.is(db.name, "library");
                tx = db.transaction("books", "readwrite");
                tx.oncomplete = function () {
                    console.log("oncomplete called");
                };
                store = tx.objectStore("books");
                store.put({ title: "Quarry Memories", author: "Fred", isbn: 123456 });
                store.put({ title: "Water Buffaloes", author: "Fred", isbn: 234567 });
                store.put({ title: "Bedrock Nights", author: "Barney", isbn: 345678 });
                return [4 /*yield*/, promiseFromTransaction(tx)];
            case 2:
                _a.sent();
                tx2 = db.transaction("books", "readwrite");
                store2 = tx2.objectStore("books");
                req1 = store2.get(234567);
                return [4 /*yield*/, promiseFromRequest(req1)];
            case 3:
                _a.sent();
                t.is(req1.readyState, "done");
                t.is(req1.result.author, "Fred");
                store2.delete(123456);
                req2 = store2.get(123456);
                return [4 /*yield*/, promiseFromRequest(req2)];
            case 4:
                _a.sent();
                t.is(req2.readyState, "done");
                t.is(req2.result, undefined);
                req3 = store2.get(234567);
                return [4 /*yield*/, promiseFromRequest(req3)];
            case 5:
                _a.sent();
                t.is(req3.readyState, "done");
                t.is(req3.result.author, "Fred");
                return [4 /*yield*/, promiseFromTransaction(tx2)];
            case 6:
                _a.sent();
                t.pass();
                return [2 /*return*/];
        }
    });
}); });
ava_1.default("export", function (t) { return __awaiter(void 0, void 0, void 0, function () {
    var backend, idb, request, db, tx, store, exportedData, backend2, exportedData2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                backend = new MemoryBackend_1.default();
                idb = new BridgeIDBFactory_1.default(backend);
                request = idb.open("library", 42);
                request.onupgradeneeded = function () {
                    var db = request.result;
                    var store = db.createObjectStore("books", { keyPath: "isbn" });
                    var titleIndex = store.createIndex("by_title", "title", { unique: true });
                    var authorIndex = store.createIndex("by_author", "author");
                };
                return [4 /*yield*/, promiseFromRequest(request)];
            case 1:
                db = _a.sent();
                tx = db.transaction("books", "readwrite");
                tx.oncomplete = function () {
                    console.log("oncomplete called");
                };
                store = tx.objectStore("books");
                store.put({ title: "Quarry Memories", author: "Fred", isbn: 123456 });
                store.put({ title: "Water Buffaloes", author: "Fred", isbn: 234567 });
                store.put({ title: "Bedrock Nights", author: "Barney", isbn: 345678 });
                return [4 /*yield*/, promiseFromTransaction(tx)];
            case 2:
                _a.sent();
                exportedData = backend.exportDump();
                backend2 = new MemoryBackend_1.default();
                backend2.importDump(exportedData);
                exportedData2 = backend2.exportDump();
                t.assert(exportedData.databases["library"].objectStores["books"].records.length === 3);
                t.deepEqual(exportedData, exportedData2);
                t.is(exportedData.databases["library"].schema.databaseVersion, 42);
                t.is(exportedData2.databases["library"].schema.databaseVersion, 42);
                t.pass();
                return [2 /*return*/];
        }
    });
}); });
//# sourceMappingURL=MemoryBackend.test.js.map